/*
    JV Library
    Copyright (C) 2014  Jouko Juhani Virtanen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
# include <iostream>
# include <vector>
# include <algorithm>

using namespace std;

# include "IOUtils.h"
# include "StringUtils.h"
# include "TypeDef.h"
# include "Structures.h"
# include "VectorManip.h"
# include "ReadGro.h"

void groLineToInfo(string line, int &residue, string &residueName, string &atomName, int &atomNum, Real &x, Real &y, Real &z)
{
	int residueEnd=5;
	int residueNameEnd=9;
	int atomNameEnd=15;
	int atomNumEnd=20;
	int xEnd=28;
	int yEnd=36;
	int zEnd=44;
	
	residue=StrToInt(line.substr(0, residueEnd));
	residueName=line.substr(residueEnd, residueNameEnd-residueEnd);
	atomName=line.substr(residueNameEnd, atomNameEnd-residueNameEnd);
	atomNum=StrToInt(line.substr(atomNameEnd, atomNumEnd-atomNameEnd));
	x=StrToReal(line.substr(atomNumEnd, xEnd-atomNameEnd))*10.0;
	y=StrToReal(line.substr(xEnd, yEnd-xEnd))*10.0;
	z=StrToReal(line.substr(yEnd, zEnd-yEnd))*10.0;
	Trim(residueName);
	Trim(atomName);
}

void groLineToAtom(string line, AtomStruct &Atom)
{
	groLineToInfo(line, Atom.ResidueNum, Atom.ResidueName, Atom.AtomName, Atom.AtomNumber, Atom.x, Atom.y, Atom.z);
}

void readGro(string GroFile, vector<AtomStruct> &Atoms)
{
	fstream file;
	string line;
	AtomStruct tempAtom;

	OpenFile(GroFile, file, "GroFile");

	getline(file, line);
	getline(file, line);
	getline(file, line);

	while (true)
	{
		if (file.eof()) break;
		if (line.length()>=44) 
		{
			groLineToAtom(line, tempAtom);
			SafePushBack(Atoms, tempAtom, "Atoms");
		}
		getline(file, line);
	}
	file.close();
	cout <<"At end of readGro Atoms.size()= "<<Atoms.size()<<endl;
}
