/*
    JV Library
    Copyright (C) 2014  Jouko Juhani Virtanen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _Structures_included_
#define _Structures_included_

# include <fstream>
# include <iostream>
# include <string>
# include <cstring>
# include <sstream>
# include <time.h>
# include <vector>
# include <iomanip>
# include <complex>
# include <numeric>

using namespace std;

# include "Constants.h"
# include "TypeDef.h"
# include "TypeDef.h"
# include "MathStructures.h"

#if _MSC_VER > 1000
#pragma once
#endif

struct VectorStruct
{
	Real x, y, z;
	VectorStruct operator + (VectorStruct);
	VectorStruct operator - (VectorStruct);
	VectorStruct operator * (Real);
	Real operator * (VectorStruct);
	VectorStruct operator / (Real);
};

class AtomStruct
{
	public:
		bool HetAtom, solute, core;
		int AtomNumber, ResidueNum;
		int atomid, AtomType, AtomType2;
		int ParmType;
		int residueid;
                int xbin, ybin, zbin;
		//int atomtype;
		vector<int> connectivity, clash;
                Real ResidencyTime;
		Real weight, Electrons, charge, mass;
		//Real partialcharge;
		Real vdw;
		Real epsilon, epsilon14, vdw14, HydrationRadius;
		Real x, y, z, Occupancy, BFactor;
		VectorStruct frac;		//Fractional coordinates
		vector<Real> anisou;
		string AtomName, CharmmAtomName, duplicate, ResidueName, ChainName, SegID, ID;
		bool operator == (AtomStruct Atom);
		bool operator != (AtomStruct Atom);
		AtomStruct();
};

struct BondAngleStruct
{
	int atom1, atom2, atom3;
	Real angle;
};

struct SymmetryOperationStruct
{
	Matrix rotationMat;
	vector<Real> translationVector;
};

struct ProteinStruct
{
	string name, bravisLatticeCentering, spaceGroup;
	vector<string> sequence;
        int FirstResidue, LastResidue, NumResidues;
	int symmetry1, symmetry2, symmetry3; //Space group
	//int numPerUnitCell;
	int numPerUnitCell, intSpaceGroup;
	vector<int> ResidueType;
	vector<int> segmentStart, segmentEnd; //For occupancy optimization
        Real XBoxLength, YBoxLength, ZBoxLength;
        Real alpha, beta, gamma;
	VectorStruct a, b, c;
	Vector center, translation;
	Matrix principalAxes, rotationMatrix; //Different from symmetry ops
	vector<Real> phi, psi, omega, dihedral;
        vector<BondAngleStruct> bondAngles;
	Matrix DistMatrix, cartToFrac, fracToCart;	
	//cartToFrac converts Cartesian coordinates to fractional coordiantes
	vector<SymmetryOperationStruct> symmetryOperations;
	vector<AtomStruct> Atoms;
	ProteinStruct& operator = (const ProteinStruct &Protein);
	ProteinStruct();
};

class CubeStruct
{
	public:
		bool angular, phobic, concave, occupied;
		Real x, y, z, density, entropy, MinDist, deg;
		int AllAtomType, AtomType, ElementType;
		int IntDist, LocatedIn, nearest, SecondNearest;
		vector<Real> IntOrientation, ResidencyTimes, ComponentDensity;
                //Real NumAtoms[NumAtomTypes];
		CubeStruct& operator = (const CubeStruct &cube);
		CubeStruct();
};

typedef vector< vector< vector<CubeStruct> > > lattice;

struct ElementStruct
{
	Real HydrationRadius, SolventCorrectedElectrons;
        vector<Real> alpha, beta;
};

struct PosStruct
{
	vector<Real> Pos;
	//bool operator == (PosStruct Pos);
};

struct DofStruct
{
	int OccupancySegments, nprot;
	vector<int> dofType;
};

struct HydrogenBondingStruct
{
	vector<int> nho1, ohn1, nho2, ohn2;
	Vector nho1Energy, ohn1Energy, nho2Energy, ohn2Energy;
};

void InitializeCube(CubeStruct &Cube);
void PrintCubeInfo(CubeStruct &Cube);
#endif
